/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.lib.gui;

import java.awt.Color;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import gnu.trove.set.hash.TIntHashSet;

import net.minecraft.client.gui.FontRenderer;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.gui.inventory.GuiContainer;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.RenderHelper;
import net.minecraft.item.ItemStack;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.MathHelper;

import buildcraft.api.core.render.ISprite;

import buildcraft.lib.BCLibSprites;
import buildcraft.lib.expression.api.IVariableNode.IVariableNodeBoolean;
import buildcraft.lib.gui.config.GuiConfigManager;
import buildcraft.lib.gui.elem.ToolTip;
import buildcraft.lib.gui.ledger.LedgerHelp;
import buildcraft.lib.gui.ledger.LedgerOwnership;
import buildcraft.lib.gui.pos.GuiRectangle;
import buildcraft.lib.gui.pos.IGuiArea;
import buildcraft.lib.gui.pos.IGuiPosition;
import buildcraft.lib.gui.pos.MousePosition;
import buildcraft.lib.misc.GuiUtil;

public abstract class GuiBC8<C extends ContainerBC_Neptune> extends GuiContainer {
    /** Used to control if this gui should show debugging lines, and other oddities that help development. */
    public static final IVariableNodeBoolean isDebuggingEnabled;
    /** If true then the debug icon will be shown. */
    private static final IVariableNodeBoolean isDebuggingShown;

    static {
        ResourceLocation debugDef = new ResourceLocation("buildcraftlib", "base");
        isDebuggingShown = GuiConfigManager.getOrAddBoolean(debugDef, "debugging_is_shown", false);
        isDebuggingEnabled = GuiConfigManager.getOrAddBoolean(debugDef, "debugging_is_enabled", false);
    }

    public static final GuiSpriteScaled SPRITE_DEBUG = new GuiSpriteScaled(BCLibSprites.DEBUG, 16, 16);

    public final C container;
    public final MousePosition mouse = new MousePosition();
    public final RootPosition rootElement = new RootPosition(this);

    /** All of the {@link IGuiElement} which will be drawn by this gui. */
    public final List<IGuiElement> shownElements = new ArrayList<>();
    public IMenuElement currentMenu;
    /** Ledger-style elements. */
    public IGuiPosition lowerLeftLedgerPos, lowerRightLedgerPos;
    private float lastPartialTicks;

    public GuiBC8(C container) {
        super(container);
        this.container = container;
        lowerLeftLedgerPos = rootElement.offset(0, 5);
        lowerRightLedgerPos = rootElement.getPosition(1, -1).offset(0, 5);

        if (container instanceof ContainerBCTile<?>) {
            shownElements.add(new LedgerOwnership(this, ((ContainerBCTile<?>) container).tile, true));
        }
        if (shouldAddHelpLedger()) {
            shownElements.add(new LedgerHelp(this, false));
        }
    }

    @Override
    public void func_73863_a(int mouseX, int mouseY, float partialTicks) {
        boolean drawEverything = currentMenu == null || !currentMenu.shouldFullyOverride();
        if (drawEverything) {
            this.func_146276_q_();
        }
        super.func_73863_a(mouseX, mouseY, partialTicks);
        if (drawEverything) {
            this.func_191948_b(mouseX, mouseY);
        }
    }

    protected boolean shouldAddHelpLedger() {
        return true;
    }

    @Override
    public void func_73866_w_() {
        super.func_73866_w_();
    }

    // Protected -> Public

    public void drawGradientRect(IGuiArea area, int startColor, int endColor) {
        int left = (int) area.getX();
        int right = (int) area.getEndX();
        int top = (int) area.getY();
        int bottom = (int) area.getEndY();
        func_73733_a(left, top, right, bottom, startColor, endColor);
    }

    @Override
    public void func_73733_a(int left, int top, int right, int bottom, int startColor, int endColor) {
        super.func_73733_a(left, top, right, bottom, startColor, endColor);
    }

    public List<GuiButton> getButtonList() {
        return field_146292_n;
    }

    public FontRenderer getFontRenderer() {
        return field_146289_q;
    }

    public float getLastPartialTicks() {
        return lastPartialTicks;
    }

    // Gui -- double -> int

    public void drawTexturedModalRect(double posX, double posY, double textureX, double textureY, double width,
        double height) {
        int x = MathHelper.func_76128_c(posX);
        int y = MathHelper.func_76128_c(posY);
        int u = MathHelper.func_76128_c(textureX);
        int v = MathHelper.func_76128_c(textureY);
        int w = MathHelper.func_76128_c(width);
        int h = MathHelper.func_76128_c(height);
        func_73729_b(x, y, u, v, w, h);
    }

    public void drawString(FontRenderer fontRenderer, String text, double x, double y, int colour) {
        fontRenderer.func_175063_a(text, (float) x, (float) y, colour);
    }

    // Other

    public List<IGuiElement> getElementsAt(double x, double y) {
        List<IGuiElement> elements = new ArrayList<>();
        IMenuElement m = currentMenu;
        if (m != null) {
            elements.addAll(m.getThisAndChildrenAt(x, y));
            if (m.shouldFullyOverride()) {
                return elements;
            }
        }
        for (IGuiElement elem : shownElements) {
            elements.addAll(elem.getThisAndChildrenAt(x, y));
        }
        return elements;
    }

    /** @deprecated Use {@link GuiUtil#drawItemStackAt(ItemStack,int,int)} instead */
    @Deprecated
    public static void drawItemStackAt(ItemStack stack, int x, int y) {
        GuiUtil.drawItemStackAt(stack, x, y);
    }

    @Override
    public void func_73876_c() {
        super.func_73876_c();
        for (IGuiElement element : shownElements) {
            element.tick();
        }
    }

    @Override
    protected final void func_146976_a(float partialTicks, int mouseX, int mouseY) {

        // FIX FOR MC-121719 // https://bugs.mojang.com/browse/MC-121719
        partialTicks = field_146297_k.func_184121_ak();
        // END FIX

        GlStateManager.func_179131_c(1, 1, 1, 1);
        if (isDebuggingShown.evaluate()) {
            SPRITE_DEBUG.drawAt(0, 0);
            if (isDebuggingEnabled.evaluate()) {
                func_73734_a(0, 0, 16, 16, 0x33_FF_FF_FF);

                // draw the outer resizing edges
                int w = 320;
                int h = 240;

                int sx = (field_146294_l - w) / 2;
                int sy = (field_146295_m - h) / 2;
                int ex = sx + w + 1;
                int ey = sy + h + 1;
                sx--;
                sy--;

                func_73734_a(sx, sy, ex + 1, sy + 1, -1);
                func_73734_a(sx, ey, ex + 1, ey + 1, -1);

                func_73734_a(sx, sy, sx + 1, ey + 1, -1);
                func_73734_a(ex, sy, ex + 1, ey + 1, -1);
            }
        }

        RenderHelper.func_74518_a();
        this.lastPartialTicks = partialTicks;
        mouse.setMousePosition(mouseX, mouseY);

        drawBackgroundLayer(partialTicks);

        for (IGuiElement element : shownElements) {
            if (element != currentMenu) {
                element.drawBackground(partialTicks);
            }
        }
    }

    @Override
    protected final void func_146979_b(int mouseX, int mouseY) {
        GlStateManager.func_179109_b(-field_147003_i, -field_147009_r, 0);
        mouse.setMousePosition(mouseX, mouseY);

        drawForegroundLayer();

        for (IGuiElement element : shownElements) {
            if (element != currentMenu) {
                element.drawForeground(lastPartialTicks);
            }
        }

        IMenuElement m = currentMenu;
        if (m != null) {
            if (m.shouldFullyOverride()) {
                GlStateManager.func_179097_i();
                func_146276_q_();
                GlStateManager.func_179126_j();
            }
            m.drawBackground(lastPartialTicks);
            m.drawForeground(lastPartialTicks);
        }

        GuiUtil.drawVerticallyAppending(mouse, getAllTooltips(), this::drawTooltip);

        if (isDebuggingEnabled.evaluate()) {
            int x = 6;
            int y = 18;
            List<String> info = new ArrayList<>();
            TIntHashSet xAxisFilled = new TIntHashSet();
            for (IGuiElement elem : this.getElementsAt(mouse.getX(), mouse.getY())) {
                String name = elem.getDebugInfo(info);
                int sx = (int) elem.getX();
                int sy = (int) elem.getY();
                int ex = sx + (int) elem.getWidth() + 1;
                int ey = sy + (int) elem.getHeight() + 1;
                sx--;
                sy--;

                int colour = (name.hashCode() | 0xFF_00_00_00);
                float[] hsb = Color.RGBtoHSB(colour & 0xFF, (colour >> 8) & 0xFF, (colour >> 16) & 0xFF, null);
                int colourDark = Color.HSBtoRGB(hsb[0], hsb[1], Math.max(hsb[2] - 0.25f, 0)) | 0xFF_00_00_00;

                func_73734_a(sx, sy, ex + 1, sy + 1, colour);
                func_73734_a(sx, ey, ex + 1, ey + 1, colour);

                func_73734_a(sx, sy, sx + 1, ey + 1, colour);
                func_73734_a(ex, sy, ex + 1, ey + 1, colour);

                func_73734_a(sx - 1, sy - 1, ex + 2, sy, colourDark);
                func_73734_a(sx - 1, ey + 1, ex + 2, ey + 2, colourDark);

                func_73734_a(sx - 1, sy - 1, sx, ey + 2, colourDark);
                func_73734_a(ex + 1, sy - 1, ex + 2, ey + 2, colourDark);

                func_73731_b(getFontRenderer(), name, x, y, -1);

                int w = getFontRenderer().func_78256_a(name) + 3;

                int mx = ((sx + 3) >> 2) << 2;
                for (int x2 = mx; x2 < ex; x2 += 4) {
                    if (xAxisFilled.add(x2)) {
                        mx = x2;
                        break;
                    }
                }

                func_73730_a(x + w, mx, y + 4, colour);
                func_73728_b(mx, y + 4, sy, colour);
                y += getFontRenderer().field_78288_b + 2;

                for (String line : info) {
                    func_73731_b(getFontRenderer(), line, x + 7, y, -1);
                    y += getFontRenderer().field_78288_b + 2;
                }
                info.clear();
            }
        }

        GlStateManager.func_179109_b(field_147003_i, field_147009_r, 0);
    }

    private List<ToolTip> getAllTooltips() {
        List<ToolTip> tooltips = new ArrayList<>();

        IMenuElement m = currentMenu;
        if (m != null) {
            m.addToolTips(tooltips);
            if (m.shouldFullyOverride()) {
                return tooltips;
            }
        }

        if (this instanceof ITooltipElement) {
            ((ITooltipElement) this).addToolTips(tooltips);
        }
        for (IGuiElement elem : shownElements) {
            elem.addToolTips(tooltips);
        }
        for (GuiButton button : getButtonList()) {
            if (button instanceof ITooltipElement) {
                ((ITooltipElement) button).addToolTips(tooltips);
            }
        }
        return tooltips;
    }

    private int drawTooltip(ToolTip tooltip, double x, double y) {
        return 4 + GuiUtil.drawHoveringText(tooltip, (int) Math.round(x), (int) Math.round(y), field_146294_l, field_146295_m, -1,
            field_146297_k.field_71466_p);
    }

    public void drawProgress(GuiRectangle rect, GuiIcon icon, double widthPercent, double heightPercent) {
        double nWidth = rect.width * Math.abs(widthPercent);
        double nHeight = rect.height * Math.abs(heightPercent);
        ISprite sprite = GuiUtil.subRelative(icon.sprite, 0, 0, widthPercent, heightPercent);
        double x = rect.x + rootElement.getX();
        double y = rect.y + rootElement.getY();
        GuiIcon.draw(sprite, x, y, x + nWidth, y + nHeight);
    }

    @Override
    protected void func_73864_a(int mouseX, int mouseY, int mouseButton) throws IOException {
        super.func_73864_a(mouseX, mouseY, mouseButton);

        mouse.setMousePosition(mouseX, mouseY);

        if (isDebuggingShown.evaluate()) {
            GuiRectangle debugRect = new GuiRectangle(0, 0, 16, 16);
            if (debugRect.contains(mouse)) {
                isDebuggingEnabled.set(!isDebuggingEnabled.evaluate());
            }
        }

        IMenuElement m = currentMenu;
        if (m != null) {
            m.onMouseClicked(mouseButton);
            if (m.shouldFullyOverride()) {
                return;
            }
        }

        for (IGuiElement element : shownElements) {
            if (element instanceof IInteractionElement) {
                ((IInteractionElement) element).onMouseClicked(mouseButton);
            }
        }
    }

    @Override
    protected void func_146273_a(int mouseX, int mouseY, int clickedMouseButton, long timeSinceLastClick) {
        super.func_146273_a(mouseX, mouseY, clickedMouseButton, timeSinceLastClick);

        mouse.setMousePosition(mouseX, mouseY);

        IMenuElement m = currentMenu;
        if (m != null) {
            m.onMouseDragged(clickedMouseButton, timeSinceLastClick);
            if (m.shouldFullyOverride()) {
                return;
            }
        }

        for (IGuiElement element : shownElements) {
            if (element instanceof IInteractionElement) {
                ((IInteractionElement) element).onMouseDragged(clickedMouseButton, timeSinceLastClick);
            }
        }
    }

    @Override
    protected void func_146286_b(int mouseX, int mouseY, int state) {
        super.func_146286_b(mouseX, mouseY, state);

        mouse.setMousePosition(mouseX, mouseY);

        IMenuElement m = currentMenu;
        if (m != null) {
            m.onMouseReleased(state);
            if (m.shouldFullyOverride()) {
                return;
            }
        }

        for (IGuiElement element : shownElements) {
            if (element instanceof IInteractionElement) {
                ((IInteractionElement) element).onMouseReleased(state);
            }
        }
    }

    protected void drawBackgroundLayer(float partialTicks) {}

    protected void drawForegroundLayer() {}

    public static final class RootPosition implements IGuiArea {
        public final GuiBC8<?> gui;

        public RootPosition(GuiBC8<?> gui) {
            this.gui = gui;
        }

        @Override
        public double getX() {
            return gui.field_147003_i;
        }

        @Override
        public double getY() {
            return gui.field_147009_r;
        }

        @Override
        public double getWidth() {
            return gui.field_146999_f;
        }

        @Override
        public double getHeight() {
            return gui.field_147000_g;
        }
    }
}
